"use strict";

var path = require('path');

var chalk = require('chalk');

var table = require('table');

var stringWidth = require('string-width');

var _ = require('lodash');

var symbols = require('log-symbols');

var utils = require('postcss-reporter/lib/util');

var MARGIN_WIDTHS = 9;
var levelColors = {
  info: 'blue',
  warning: 'yellow',
  error: 'red'
};

function logFrom(fromValue) {
  if (fromValue.charAt(0) === '<') {
    return fromValue;
  }

  return path.relative(process.cwd(), fromValue).split(path.sep).join('/');
}

var columnWidths = {
  0: 1,
  1: 1,
  2: 1,
  3: 1,
  4: 1
};

var calculateWidths = function calculateWidths(columns) {
  _.forOwn(columns, function (value, key) {
    var normalisedValue = value ? value.toString() : value;
    columnWidths[key] = Math.max(columnWidths[key], stringWidth(normalisedValue));
  });

  return columns;
};

function getMessageWidth(widths) {
  if (!process.stdout.isTTY) {
    return widths[3];
  }

  var availableWidth = process.stdout.columns < 80 ? 80 : process.stdout.columns;

  var fullWidth = _.sum(_.values(columnWidths)); // If there is no reason to wrap the text, we won't align the last column to the right


  if (availableWidth > fullWidth + MARGIN_WIDTHS) {
    return columnWidths[3];
  }

  return availableWidth - (fullWidth - columnWidths[3] + MARGIN_WIDTHS);
}

module.exports = function (_ref) {
  var source = _ref.source,
      messages = _ref.messages;
  var output = '\n';

  if (source) {
    output = "".concat(output).concat(chalk.underline(logFrom(source)), "\n");
  }

  var orderedMessages = _.sortBy(messages, function (m) {
    return m.line ? 2 : 1;
  }, // positionless first
  function (m) {
    return m.line;
  }, function (m) {
    return m.column;
  });

  var cleanedMessages = orderedMessages.map(function (message) {
    var location = utils.getLocation(message);
    var severity = message.severity;
    var row = [location.line || '', location.column || '', symbols[severity] ? chalk[levelColors[severity]](symbols[severity]) : severity, message.text // Remove all control characters (newline, tab and etc)
    .replace(/[\x01-\x1A]+/g, ' ') // eslint-disable-line
    .replace(/\.$/, '').replace(new RegExp("".concat(_.escapeRegExp("(".concat(message.rule, ")")), "$")), ''), chalk.dim(message.rule || '')];
    calculateWidths(row);
    return row;
  });
  output += table.table(cleanedMessages, {
    border: table.getBorderCharacters('void'),
    columns: {
      0: {
        alignment: 'right',
        width: columnWidths[0],
        paddingRight: 0
      },
      1: {
        alignment: 'left',
        width: columnWidths[1]
      },
      2: {
        alignment: 'center',
        width: columnWidths[2]
      },
      3: {
        alignment: 'left',
        width: getMessageWidth(columnWidths),
        wrapWord: true
      },
      4: {
        alignment: 'left',
        width: columnWidths[4],
        paddingRight: 0
      }
    },
    drawHorizontalLine: function drawHorizontalLine() {
      return false;
    }
  }).split('\n').map(function (el) {
    return el.replace(/(\d+)\s+(\d+)/, function (m, p1, p2) {
      return chalk.dim("".concat(p1, ":").concat(p2));
    });
  }).join('\n');
  return output;
};